@@ -1,9 +1,9 @@
-package Parse::RecDescent::FAQ;
+package Parse::RecDescent::FAQ; 
 
 use vars qw($VERSION);
 
-our $VERSION = sprintf '%s', q$Revision: 2.8 $ =~ /Revision:\s+(.*)\s+/ ;
 
+our $VERSION = sprintf '%s', q$Revision: 3.1 $ =~ /Revision:\s+(\S+)\s+/ ;
 
 1;
 __END__
@@ -12,6 +12,353 @@ __END__
 
 Parse::RecDescent::FAQ - the official, authorized FAQ for Parse::RecDescent. 
 
+=head1 OVERVIEW-TYPE QUESTIONS
+
+=head2 Is Parse::RecDescent LL(1)? LL(N)? LR(1)? LR(N)?
+
+=over 4
+
+=item Answer by Yves Orton:
+
+I have a data structure which is
+
+a hash of entries
+where
+an entry is a list/array of sets
+
+I have also a grammar that can parse the syntax of the text files that
+contain the data I want to fill this structure with. Until here
+everything is ok.
+
+Problem: I cannot figure out how to actually FILL the parsed data into
+the structure. I can only decide if a string is grammatically correct
+or not.
+
+
+
+
+=back
+
+Also see the "Left-recursion" section under "PARSER BEHAVIOR"
+
+=head1 DEBUGGING
+
+Paraphrased from Yves Orton:
+
+To understand more about why your parser is not behaving as it should
+take advantage of the follow variables:
+
+   $::RD_ERRORS       # unless undefined, report fatal errors
+   $::RD_WARN         # unless undefined, also report non-fatal problems
+   $::RD_HINT         # if defined, also suggestion remedies
+   $::RD_TRACE        # if defined, also trace parsers' behaviour
+
+
+
+=head1 PARSER BEHAVIOR
+
+=head2 Insuring a top-level rule match
+
+I have a question regarding subrules within grammars used with
+Parse::RecDescent - The following code best illustrates the best test
+case that I can identify which highlights my problem. 
+
+
+
+
+
+Note that with this test code, each of the lines within the __DATA__ section are tested against the grammar and the expected result, either pass or fail, is indicated by the 1 or 0 at the start of the line respectively.
+
+However, despite these expected results, the grammar does not reject
+the lines arg1, and arg1,arg2,, which ideally should be rejected due
+to the incomplete match of the subrule (comma element) - In these
+cases, a trace shows that the subrule terminal comma is matched and
+consumed, despite the entire subrule, consisting of comma and element,
+not being matched. I am sure this is a relatively straight-forward
+oversight within the grammar on my part, but I am at a loss as to how
+to correct this.  
+
+Program fragment delivered error ``couldnt open file : No such file or directory at /home/metaperl/bin/tt.pl line 18, <F> line 14.''
+
+=head3 Answer by Randal L. Schwartz
+
+It's matching a portion of the string, which is legal unless you also
+anchor the end of the pattern. I typically use C</\z/> at the end of my
+top-level pattern.  
+
+Also, your comma-separated string can be parsed simply with 
+
+ argument: element(s /,/)
+
+as shown in the P::RD examples on the manpage. 
+
+=head2 Backtracking (PRD doesn't do it)
+
+Take the following regex:
+
+  /(aa|a)a/
+
+It will match this text:
+
+  aaa
+
+Now take the following PRD grammar:
+
+ use strict;
+ use warnings;
+ use Parse::RecDescent;
+ 
+ my $grammar = 'startrule: ( "aa" | "a" ) "a"';
+ my $parser  = Parse::RecDescent->new($grammar);
+ my $text    = 'aa';
+ 
+ print defined($parser->startrule($text)) ? "Good!\n" : "Bad!\n";
+
+It *will* print "Bad", meaning that PRD did not match it.
+
+The reason is that the first branch of the alternation matched, then the next subrule failed.
+
+You may ask: is there any way to persuade a top-down parser like P::RD
+to accept the above text as valid? I know that, in this simple
+example, I could easily rewrite the grammar (either by saying 
+
+ ("a" |"aa" ) "a" 
+
+or 
+
+ "aa" a" | "a" "a"
+
+but what I mean is: Is there any
+additional feature I have missed which would allow the grammar as is
+to parse the text successfully? 
+
+To put the question another way, can I get P::RD to behave more like a
+regex engine? After all, even an NFA engine would backtrack to try all
+possible alternatives before failing :-) (Perhaps parsers just do not
+backtrack past individual subrules under any circumstances.) 
+
+=head3 And the answer is...
+
+RecDescent parsers do not work that way. They don't backtrack on
+failure; they just fail. Of course, there's nothing to prevent a
+recursive descent parser from incorporating backtracking too, but
+RecDescent doesn't. 
+
+So, if you need backtracking in part of your grammar, you need to use
+plain old regexes there.
+
+
+
+
+=head2 Skipping over things
+
+The variable
+
+  $Parse::RecDescent::skip 
+
+is what you need. It takes strings and C<qr>-quoted regular expressions.
+See the Parse::RecDescent docs for details.
+
+Also, see the C<Terminal Separators> parts of the docs.
+
+Here is an example of removing C comments in program text
+(courtesy Damian Conway) by treating them as whitespace!
+
+	program: <skip: qr{\s* (/[*] .*? [*]/ \s*)*}x> statement(s)
+
+	statement: # etc...
+
+=head3 Quoting within a skip expression:
+
+Here is my script:
+
+ ------ Start Script ------
+ use strict;
+ use warnings;
+ 
+ $::RD_TRACE = 1;
+ 
+ use Parse::RecDescent;
+ 
+ my $grammar = q{
+ 
+    input:  number(s) { $return = $item{ number } } | <error>
+ 
+    number: <skip: '\.*'> /\d+/ 
+ 
+ };
+
+ my $parser = new Parse::RecDescent($grammar);
+ 
+ my $test_string = qq{1.2.3.5.8};
+
+ print join( "\n", @{ $parser -> input( $test_string ) } );
+ ------ End Script ------
+
+This script works great. However, if I change the value of the skip
+directive so that it uses double quotes instead of single quotes:
+
+ <skip: "\.*">
+
+the grammar fails to parse the input. However, if I put square
+brackets around the escaped dot:
+
+ <skip: "[\.]*">
+
+the grammar starts working again...
+
+=head4 and here's why:
+
+This small test program may help you figure out what's going wrong:
+
+	print "\.*", "\n";
+	print '\.*', "\n";
+
+Backslash works differently inside single and double quotes.
+Try:
+
+      <skip: "\\.*">
+
+The reason the third variant:
+
+      <skip: "[\.]*">
+
+works is because it becomes the pattern:
+
+	/[.]/
+
+which is a literal dot.
+
+=head2 Left-recursion
+
+=over 4
+
+=item * On elimination of left-recursion Randal Schwartz states:
+
+I had a fun time eliminating a mutual left-recursion problem for
+my "Data-Undumper" program, which used P::RD as an essential component.
+See my discussion of such at
+
+ http://www.stonehenge.com/merlyn/LinuxMag/col29.html
+
+=item * Also, regarding elimination see this Perlmonks node:
+
+ http://www.perlmonks.org/index.pl?lastnode_id=6364&node_id=153155
+
+=item * Regarding detection of left-recursion, Conway states:
+
+RecDescent does a complete graph traversal looking for n-ary
+left-recursion loops and fails to compile the grammar if it finds any
+left-recursive loop involving any number of rules. It has done this since its
+earliest versions.
+
+=back
+
+
+
+
+=head2 Commit in subrule which is optional in rule
+
+Ques: When a subrule in a rule that has a "zero or
+more" repetition specifier (ie. ? or s?) has a
+<commit> directive in its production followed by the
+conditional <error?> <reject> production, if that
+subrule's production becomes committed, does that
+error cause the rule containing the subrule to fail
+also?  It should right, if we have committed?  It does
+not seem to work.
+
+Here is what I mean:
+
+ myrule: 'stuff' mysubrule(?)
+
+ mysubrule: ID <commit> '[' ']'
+       | <error?> <reject>
+
+If the 1st production of mysubrule has committed, then
+myrule should fail.  It doesn't seem to.  If this is
+not a bug, how do I get this behavior?
+
+=over 4
+
+=item * Answer by Damian
+
+The optional nature of the reference to mysubrule(?)
+means that, when the subrule fails (whether committed or not)
+the failure doesn't matter, since myrule can match if it
+finds zero mysubrules, which it just did.
+
+
+The usual way to get the rule-of-the-subrule to fail upon subrule failure is
+by "anchoring" the end of the match. That might be:
+
+  myrule: 'stuff' mysubrule(?) ...!ID
+
+  mysubrule: ID <commit> '[' ']'
+           | <error?> <reject>
+
+or 
+
+  myrule: 'stuff' mysubrule(?) /\s*\Z/
+
+  mysubrule: ID <commit> '[' ']'
+           | <error?> <reject>
+
+
+or whatever addition confirms that there really wasn't anything else
+after 'stuff'.
+
+=item * Now that you think you know the answer...
+
+That answer is partially wrong, as was pointed out by Marcel Grunaer.
+In this phrase:
+
+  myrule: 'stuff' mysubrule(?) ...!ID
+
+it is necessary to return a { 1 }, as the rule
+fails otherwise, presumably because of the negative lookahead:
+
+  myrule: 'stuff' mysubrule(?) ...!ID { 1 }
+
+Marcel went on to point out an optimization:
+
+another option would be the use of a rulevar:
+
+  myrule : <rulevar: local $failed>
+  myrule : 'stuff' mysubrule(?) <reject:$failed>
+
+  mysubrule: ID <commit> '[' ']'
+    | <error?> { $failed++ }
+
+this way you don't have to specify a potentially complex negative
+lookahead, and the method works over several levels of subrules
+as well.
+
+
+=back
+
+
+=head2 Making warning line numbers correspond to your grammar
+
+How do I match the line numbers with the actual contents of my
+script?
+
+At present, you can't (but that's on the ToDo list).
+Setting C<$::RD_TRACE> can be useful though:
+
+Once you've run with C<$RD_TRACE>, do this:
+
+        perl -w RD_TRACE
+
+Then go and examine the actual line numbers given for the error
+in the file C<RD_TRACE>.
+
+That will show you the actual generated code that's the problem.
+
+That code will, in turn, give you a hint where the problem is in the
+grammar (e.g. find out which subroutine it's in, which will tell you the
+name of the offending rule).
+
 =head1 IGNORABLE TOKENS 
 
 =head2 Removing C comments
@@ -35,6 +382,51 @@ Do something like this:
 
 =head1 NEWLINE PROCESSING
 
+=head2 Parsing Windows Init Files
+
+I'm trying ot use Parse::RecDescent to parse a configuration file which
+looks like this:
+
+ [SECTION]
+ parameter 1=value 1
+ parameter2=value 2
+
+ [NEXTSECTION]
+ other parameter=other value
+
+=over 4
+
+=item * Answer by Damian
+
+       q{
+                config:
+                          section(s)
+
+                section:
+                          <skip: ''>  section_label parameter(s)
+                        | <error>
+
+                section_label:
+                          /\[[A-Z]+\]\s*/
+
+                parameter:
+                          key "=" value
+
+                key:
+                          /[A-Za-z0-9 ]+/
+
+                value:
+                          /.*\n/
+        }
+
+=item * Another Answer
+
+Also take a look at the example in section 11.2 in "Data Munging with Perl"
+by Dave Cross.
+
+=back
+
+
 =head2 As end of line
 
 
@@ -174,6 +566,93 @@ So if you would like to state the test in the positive, then do this:
 
 =head1 MODULAR / GENERATIVE / CREATIVE / HAIRY PARSING
 
+=head2 Macro processing
+
+In a RecDescent grammar, I'd like to add support in my language for
+"macros".  These macros would be very much like #define macros in C
+and would support arguments that are instantiated into the
+expansion. In C, this looks something like:  
+
+ #define   add(a,b)        (a+b)
+
+In my language, I'd like something like: 
+
+ myMacro(x,y):        any_expression
+
+where "any_expression" is any old (deep) expression defined elsewhere
+in the grammar, and C<(x,y)> would be replaced/instantiated in that
+expression.  
+This may seem straighforward, but, well, it ain't to me. (I am, after
+all, mush4brains.)  In my P::RD grammar, the "macro" rule without
+arguments would look like: 
+
+ macro:         identifier ':' expr
+
+where, again, expr is defined fully in the grammar. Adding arguments
+to this... I need to instantiate my C<(x,y)> into the "expr" string
+before the subrules interpret it.  
+Now... I thought of using standard Perl substitution in a pre-scan,
+but this seems less-than-ideal, since it requires some knowledge of
+the grammar in the Perl REs. I thought of using two separate grammars,
+the first of which is a simplified version of the full second grammar,
+but this also seems a bit redundant.  
+
+=over 4 
+
+=item * And the answer is...
+
+ http://perlmonks.org/index.pl?node_id=176399
+
+=back
+
+
+=head2 Cloning parsers but giving each parser it own package
+
+It seems that the namespace for pre-compiled parsers
+(ie. compiled into a perl module) have hard-coded
+namespaces (IE. namespace00001).  I was trying to
+clone one of these parsers by calling its 'new'
+method, but each parser is sharing the same namespace
+and thus any global variables I have in that namespace
+within a startcode block of my grammar before the
+first rule get overwritten by the other corresponding
+parser.
+
+=over 4
+
+=item * Answer by Yves Orton
+
+Heres the required patch 
+
+ # Line 1692 of Parse::RecDescent reads
+
+ my $nextnamespace = "namespace000001";
+
+ # Add This Method
+ sub set_namespace {
+   local $_=$_[1];
+   croak "$_ is not a legal namespace." if /[^\w]/;
+   $_.="00" unless /\d+$/; # Ensure our namespace has a number at the end
+   $nextnamespace=$_;
+ }
+
+ # And then just call
+ Parse::RecDescent->set_namespace("MyNameSpace");
+
+
+=back
+
+
+
+=head2 Parsing sentences to generate sentences
+
+In this column, Randal shows how to read text to generate more text.
+He parses sentences to make Parse::RecDescent parse trees which he then
+re-walks with random weightings to create new sentences.
+
+ http://www.stonehenge.com/merlyn/LinuxMag/col04.html
+
+
 =head2 Calling a parser within a grammar
 
 I have a script that uses Parse::RecDescent, in which I want to define 2
@@ -305,6 +784,96 @@ with a grammar that's way more elegant and efficient...)
  
  term : ident '(' meaning ')' | ident
 
+=back
+
+=head2 XOR as opposed IOR alternation matching
+
+I'm using alternations in some productions but in contrast to
+the definition of the |-operator, I'm looking for a behaviour
+which is XOR (^) not OR (|). So far I used the <reject>
+directive to simulate such a behaviour.
+
+Is there any easy solution to this?
+
+=over 4
+
+=item * Answer by Randal Schwartz
+
+Use a set.
+
+ use Parse::RecDescent;
+ use Data::Dumper; $|++;
+ my $parser = Parse::RecDescent->new(q{
+ 
+ line: word(s) /\z/ {
+ my @words = @{$item[1]};
+ my %count;
+ (grep ++$count{$_} > 1, @words) ? undef : \@words;
+ }
+ 
+ word: "one" | "two" | "three"
+ 
+ }) or die;
+ 
+ for ("one two", "one one", "two three one", "three one two one") {
+   print "$_ =>\n";
+   print Dumper($parser->line($_));
+ }
+ 
+ # which generates:
+ 
+ one two =>
+   $VAR1 = [
+ 	   'one',
+ 	   'two'
+ 	  ];
+ one one =>
+   $VAR1 = undef;
+ two three one =>
+   $VAR1 = [
+ 	   'two',
+ 	   'three',
+ 	   'one'
+ 	  ];
+ three one two one =>
+   $VAR1 = undef;
+
+
+
+=item * Embellishment by Damian Conway
+
+
+
+Furthermore, if uniqueness was something you needed to enforce more
+widely
+in your grammar, you could factor it out into a parametric rule:
+
+        use Parse::RecDescent;
+        use Data::Dumper; $|++;
+        my $parser = Parse::RecDescent->new(q{
+
+        line: unique['word'] /\z/ { $return = $item[1] }
+
+        unique: <matchrule: $arg[0]>(s)
+            {
+              my %seen;
+              foreach (@{$item[1]}) { undef $item[1] and last if
+$seen{$_}++ }
+              $return = $item[1];
+            }
+
+        word: "one" | "two" | "three"
+
+        }) or die;
+
+        for ("one two", "one one", "two three one", "three one two
+one") {
+          print "$_ =>\n";
+          print Dumper($parser->line($_));
+        }
+
+
+
 =back
 
 
@@ -317,7 +886,7 @@ Joseph Hall's "Effective Perl Programming"
 =head2 Use repetition modifiers with a separator pattern to match
 CSV-like data
 
-The intuitive to match CSV data is this:
+The intuitive way to match CSV data is this:
 
  CSVLine:
       NonFinalToken(s?) QuotedText
@@ -361,7 +930,8 @@ But, if speed is what you want then the second one is the way to go.
 
 =head2 Precompiling Grammars for Speed of Execution
 
-Take a look at Parse::RecDescent's precompilation option
+Take a look at Parse::RecDescent's precompilation option under the section
+titled "Precompiling parsers".
 
 =head2 Parse::RecDescent is slow on Really Big Files. How can I speed it up?
 
@@ -539,6 +1109,49 @@ $return variable in the actions.
 
 =head1 THINGS NOT TO DO
 
+=head2 Don't think that rule: statement and rule: statement(1) are the
+same
+
+Even though in pure Perl, the repetition modifier returns the same
+data structure without or without an argument of one:
+
+ use Data::Dumper;
+
+ my @a = (x);
+ my @b = (x) x 1;
+
+ my $x = 'x';
+ my $y = 'x' x 1;
+
+In this first case below, C<rule_one> returns a scalar upon matching, 
+while C<rule_two> returns an arrayref with 1 element upon matching:
+
+ rule_one: statement
+ rule_two: statement(1)
+
+However counter-intuitive this may at first sound, Damian provides us with 
+some insight:
+
+I don't think C<x> is the right analogy for RecDescent repetition operators.
+The behaviour of C<*> and C<+> in regexes is a closer model.
+
+I guess it depends on your poitn of view. I would have said that the
+absolute consistency with which *every* repetition (regardless of its
+number) returns an array ref is better than the alternative:
+
+        christmas:
+                gold_rings(5)                   # returns array ref
+                calling_birds(4)                # returns array ref
+                French_hens(3)                  # returns array ref
+                turtle_doves(2)                 # returns array ref
+                partridge_in_a_pear_tree(1)     # returns scalar value
+
+Especially if you have to change the count at some later point, which 
+would mess up any code relying on the type of value returned.
+
+
+
+
 =head2 Do not follow <resync> with <reject> to skip errors
 
 C<resync> is used to allow a rule which would normally fail to "pass" so that 
@@ -559,6 +1172,18 @@ To get each value for the number subrule, you have a couple of choices,
 both documented in the Parse::RecDescent manpage under
 C<@item and %item>.
 
+=head2 use <rulevar: local $x> not <rulevar: $x> 
+
+If you say:
+
+	somerule: <rulevar: $x>
+
+you get a lexical $x within the rule (only). If you say:
+
+	somerule: <rulevar: local $x>
+
+you get a localized $x within the rule (and any subrules it calls).
+
 =head2 Don't use $::RD_AUTOACTION to print while you are parsing
 
 You can't print out your result while you are parsing it, because you
@@ -568,7 +1193,192 @@ Instead, have the final top-level rule do all the diagnostic printing, or
 alternatively use P::RD's tracing functionality to observe parsing in 
 action.
 
+=head1 ERROR HANDLING
 
+=head2 Propagating a failure up after a <commit> on a subrule
+
+See L<Commit in subrule which is optional in rule|Commit in subrule which is optional in rule>
+
+
+
+=head2 In a non-shell (e.g. CGI) environment
+
+I need to parse a file with a parser that I cooked up from
+Parse::Recdescent. 
+My problem, it must work in a cgi environment and the script must be
+able to handle errors. I tried eval, piping, forking, Tie::STDERR, but
+the errors msgs from Parse::Recdescent seem unstoppable.
+
+I can catch them only by redirecting the script's STDERR from the
+shell. But howdo I catch them from within ??
+
+=over 4 
+
+=item * Like this:
+
+        use Parse::RecDescent;
+        open (Parse::RecDescent::ERROR, ">errfile")
+                or die "Can't redirect errors to file 'errfile'";
+
+        # your program here
+
+=back
+
+=head2 Accessing error data
+
+What I want to do is export a <error: foo bar> condition to the
+calling
+program.  Currently _error writes the error message to (what
+essentially
+is STDOUT) which means a parsing error prints a nice message, but it
+is
+up to the reader to DO anything about it.
+
+I have a Tk viewer that will parse the file.  When the parse fails, I
+would like to capture the line number and error message returned and
+position the viewer (a text widget) to that line, highlighted.  And no
+STDOUT message.
+
+Something kind of like the eval function where the return value is the
+result but $@ is set as a "side effect".
+
+Am I missing something about the built in capability?  Is the solution
+as simple as overloading the Parse::RecDescent::_error subroutine with
+my own copy which might look like this:
+
+%Parse::RecDescent::ERROR=();
+...
+sub Parse::RecDescent::_error($;$)
+{
+	$ERRORS++;
+	return 0 if ! _verbosity("ERRORS");
+	%Parse::RecDescent::ERROR=('line'=>$_[1],'msg'=>$_[0]);
+	return 1;
+}
+
+It seems like it should work and I tried it and it did.  BUT this is
+an
+extremely complex bit of code and I'm concerned about unforseen
+consequences.
+
+Of course, I could make ERROR a my variable and provide a method to
+access it so it is not a "global", and this would be OO code of a
+higher
+purity (or something), but that is not really the point.
+
+=over 4
+
+=item * Answer by Damian Conway
+
+You can get access to the error data by referring to the attribute
+C<$thisparser->{errors}> within a rule. 
+
+C<$thisparser->{errors}> is a reference to an array of arrays. Each of
+the inner arrays has two elements: the error message, and the line
+number. 
+
+So, for example, if your top-level rule is:
+
+ start: subrule1 subrule2
+
+then you could intercept any errors and print them to a log file, like
+so: 
+
+ start: subrule1 subrule2
+      | { foreach (@{$thisparser->{errors}}) {
+              print LOGFILE "Line $_->[1]:$_->[0]\n";
+          }
+          $thisparser->{errors} = undef;
+        }
+
+Note that the line:
+
+ $thisparser->{errors} = undef;
+
+is doing double duty. By resetting C<$thisparser->{errors}>, it's
+preventing those annoying error messages from being automagically
+printed. And by having the value C<undef> as the last value of the
+action, it's causing the action to fail, which means the second
+production fails, which means the top rule fails, which means errors
+still cause the parse to fail and return C<undef>.
+
+=back
+
+=head2 Simple Error Handling
+
+I'm trying to write a parser for orders for Atlantis (PBEM game).
+Syntax is pretty simple: one line per command, each command
+starts with name, followed by list of parameters. Basically it's
+something like this (grammar for parsing one line):
+
+ Statement:Comment | Command Comment(?)
+ Comment:/;.*/ 
+ Command:'#atlantis' <commit> FactionID String
+    Command:'attack' <commit> Number(s)
+ ....
+
+However I have problems to make it work as I want:
+
+1) In case of failed parsing (syntax error, not allowey keyword, ...) 
+I want to store error messages in variable (not to be just printed), so I can 
+process them later.
+
+I don't think Parse::RecDescent has a hook for that (Damian, something
+for the todo list?), but you can always install a $SIG {__WARN__}
+handler and process the generated warnings.
+
+2) In case if user types "attack bastards" I want to give him
+error message that "list of numbers expected" instead
+of just saying the "cannot parse this line". The only
+thing that I came up with now was defining every command
+like this:
+ Command:Attack
+ Attack:'attack' AttackParams
+ AttackParams:Number(s) | <error>
+ ...
+Any better solutions?
+
+=over 4
+
+=item * You can just do:
+
+    Command:   '#atlantis' <commit> FactionID String
+       |   'attack' <commit> Number(s)
+       |   <error>
+
+and when you try to parse "attack bastards", you will get:
+
+    ERROR (line 1): Invalid Command: Was expecting Number but found
+        "bastards" instead
+
+You might want to use <error?>, which will only print the error when
+it saw '#atlantis' or 'attack' (because then you are committed).
+
+=back
+
+=head2 Collecting all error messages for processing
+
+Is there a way to to collect the parser-generated errors and use them later 
+on in my script?
+
+=over 4
+
+=item * Answer by Damian
+
+There's no "official" way (yet). A future release will allow you
+to create pluggable OO error handlers. At the moment the best 
+you can do is:
+
+	startrule: try_this
+		 | try_that
+		 | try_the_other
+		 | { ::do_something_with( $thisparser->{errors} ) }
+
+$thisparser->{errors} will contain a reference to an array-of-arrays,
+where each inner array contains the error message and the line number
+it occurred at.
+
+=back
 
 =head1 OTHER Parse::RecDescent QUESTIONS
 
@@ -815,57 +1625,6 @@ do what you want).
 =back
 
 
-=head2 Error handling
-
-I'm trying to write a parser for orders for Atlantis (PBEM game).
-Syntax is pretty simple: one line per command, each command
-starts with name, followed by list of parameters. Basically it's
-something like this (grammar for parsing one line):
-
- Statement:Comment | Command Comment(?)
- Comment:/;.*/ 
- Command:'#atlantis' <commit> FactionID String
-    Command:'attack' <commit> Number(s)
- ....
-
-However I have problems to make it work as I want:
-
-1) In case of failed parsing (syntax error, not allowey keyword, ...) 
-I want to store error messages in variable (not to be just printed), so I can 
-process them later.
-
-I don't think Parse::RecDescent has a hook for that (Damian, something
-for the todo list?), but you can always install a $SIG {__WARN__}
-handler and process the generated warnings.
-
-2) In case if user types "attack bastards" I want to give him
-error message that "list of numbers expected" instead
-of just saying the "cannot parse this line". The only
-thing that I came up with now was defining every command
-like this:
- Command:Attack
- Attack:'attack' AttackParams
- AttackParams:Number(s) | <error>
- ...
-Any better solutions?
-
-=over 4
-
-=item * You can just do:
-
-    Command:   '#atlantis' <commit> FactionID String
-       |   'attack' <commit> Number(s)
-       |   <error>
-
-and when you try to parse "attack bastards", you will get:
-
-    ERROR (line 1): Invalid Command: Was expecting Number but found
-        "bastards" instead
-
-You might want to use <error?>, which will only print the error when
-it saw '#atlantis' or 'attack' (because then you are committed).
-
-=back
 
 
 =head2 How can I get at the text remaining to be parsed?
@@ -945,6 +1704,27 @@ to this:
 
 =back
 
+=head1 REGULAR EXPRESSIONS
+
+=head2 Shortest match instead of longest match
+
+What is the Perl idiom for getting the leftmost shortest match?
+For instance, so:
+
+ $a = "banana";
+ $a =~ /b.*n/;  # but different
+ print $&;
+
+would yield "ban" instead of "banan"?
+
+=over 4
+
+=item *
+
+    $a =~ /b.*?n/;
+
+=back
+
 
 
 =head1 Programming Topics Germane to Parse::RecDescent Use
@@ -1389,14 +2169,39 @@ is by far the likeliest explanation.
 
 
 
-=head1 SEE ALSO
+=head1 RESOURCES
 
 =over 4
 
+=item * MySql to Oracle schema conversion utility
+
+Written in Parse::RecDescent by Tim Bunce:
+
+ http://groups.google.com/groups?q=recdescent&start=40&hl=en&scoring=d&rnum=41&selm=9km7b1%246vc%241%40FreeBSD.csie.NCTU.edu.tw
+
+=head1 Tutorial on positive and negative lookahead and lookbehind
+regexps:
+
+ http://lists.isb.sdnpk.org/pipermail/comp-list/2002-August/001156.html
+
+=item * Data::MultiValuedHash on search.cpan.org
+
+Transparent manipulation of single or multiply-valued Perl hash values.
+
 =item * Parse::Recdescent tutorial at www.perl.com
 
 http://www.perl.com/pub/a/2001/06/13/recdecent.html
 
+=item * "Safe undumping"
+
+In this Linux Magazine article by Randal Schwartz uses
+Parse::RecDescent to parse Data::Dumper output. Not fast, but quite
+complete. 
+
+=item "Parsing Interesting Things", SysAdminMag.COM, December 2001, Randal Schwartz
+
+
+
 =item * py
 
 py, which I assume is short for C<parse yacc>, is a program by
@@ -1409,15 +2214,24 @@ It is obtainable from http://perl.plover.com/py/
 A bottom-up parser which will be familiar to those who
 have used Lex and Yacc. Parse::RecDescent is a top-down parser.
 
-=item * Text::Balanced 
+=item * Text::Balanced  and Regexp::Common::balanced
 
 Use this instead of writing hairy regular expressions to match certain
 common "balanced" forms of text, such as tags and parenthesized text.
 
+See http://perlmonks.org/index.pl?node_id=208285
+for an example of where it would be better suited for the task than
+Parse::RecDescent.
+
 =item * "Mastering Regular Expressions" by Jeffrey Freidl
 
 You still need to know when to use /.*/ or /.+/ or /[^x]*/
 
+=item * "Data Munging with Perl" by Dave Cross
+
+Chapter 11 is focused on parsing with special emphasis on practical use of
+Parse::RecDescent.
+
 =item * "Object-Oriented Perl" by Damian Conway
 
 This book will aid you in complexity management for large grammars.
@@ -1456,4 +2270,28 @@ The (unwitting) contributors to this FAQ
 
 =back 
 
+=head1 SOURCE for FAQ MATERIAL
+
+I try to regularly scan all sources of Perl question-answer for FAQ material.
+If you can think of another source that is not listed below, I would
+appreciate knowing about it.
+
+=over 4
+
+=item * comp.lang.perl.modules
+
+=item * comp.lang.perl.moderated
+
+=item * recdescent@perl.org
+
+Conveniently available via NNTP at:
+
+     news://nntp.perl.org/perl.recdescent
+
+=item * perlmonks.org
+
+  
+
+=back
+
 =cut
